@@ -1,4 +1,4 @@
-# $Id: netdisco.pm,v 1.75 2006/11/28 19:19:04 fenner Exp $
+# $Id: netdisco.pm,v 1.103 2009/10/15 14:10:38 olly_g Exp $
 
 =head1 NAME
 
@@ -29,13 +29,13 @@ use Digest::MD5;
 
 use vars qw/%DBH $DB %CONFIG %GRAPH %GRAPH_SPEED $SENDMAIL $SQLCARP %PORT_CONTROL_REASONS $VERSION/;
 @netdisco::ISA = qw/Exporter/;
-@netdisco::EXPORT_OK = qw/insert_or_update getip hostname sql_do has_layer
-                       sql_hash sql_column sql_rows sql_query add_node add_arp add_nbt dbh sql_match
+@netdisco::EXPORT_OK = qw/insert_or_update getip hostname sql_do sql_begin sql_commit sql_rollback sql_disconnect has_layer
+                       sql_hash sql_column sql_columns sql_rows sql_query add_node add_arp add_nbt dbh sql_match
                        all config updateconfig sort_ip sort_port sql_scalar root_device log
                        make_graph is_mac user_add user_del mail is_secure in_subnet in_subnets
-                       active_subnets dump_subnet in_device
-                       url_secure mask_to_bits bits_to_mask dbh_quote sql_vacuum
-                       tryuse homepath/;
+                       active_subnets dump_subnet in_device get_community
+                       url_secure is_private cidr mask_to_bits bits_to_mask dbh_quote sql_vacuum
+                       tryuse homepath user_ldap_verify ldap_search/;
 
 %netdisco::EXPORT_TAGS = (all => \@netdisco::EXPORT_OK);
 
@@ -132,7 +132,7 @@ Reason why a port would be shutdown. These get fed into C<port_control_log>
 
 =cut
 
-$VERSION = '0.95';
+$VERSION = '1.0';
 
 =back
 
@@ -183,15 +183,15 @@ sub add_node {
     my $oui = substr($mac,0,8);
     # Set the active flag to false to archive all other instances
     #   of this mac address
-    my $other = sql_do(qq/UPDATE node SET active = 'f' WHERE mac = '$mac' AND switch != '$ip'/);
+    my $other = sql_do(qq/UPDATE node SET active = 'f' WHERE mac = '$mac' AND active AND NOT (switch = '$ip' AND port = '$port')/);
 
     # Add this entry to node table. 
     my %hash = ('switch' => $ip, 'mac' => $mac, 'port' => $port );
-    my %set = ('time_last' => scalar(localtime), 'active' => 1, 'oui' => $oui);
+    my %set = ('time_last' => 'now', 'active' => 1, 'oui' => $oui);
     # if there was another node, set time_recent too.
     # NOTE: $other might be "0E0", so be careful how you test it.
     if ($other != 0) {
-	$set{time_recent} = scalar(localtime);
+        $set{time_recent} = 'now';
     }
     insert_or_update('node', \%hash, { %set, %hash });
 }
@@ -212,8 +212,8 @@ sub add_nbt {
 
     # Set the active flag to false to archive all other instances
     #   of this MAC address
-    sql_do(sprintf("UPDATE node_nbt SET active = 'f' WHERE mac=%s",
-            $dbh->quote($mac)));
+    #sql_do(sprintf("UPDATE node_nbt SET active = 'f' WHERE mac=%s",
+    #        $dbh->quote($mac)));
 
     # Add this entry to node_nbt table. 
     insert_or_update('node_nbt', { 'mac' => $mac },
@@ -235,6 +235,65 @@ sub bits_to_mask {
     return join(".",unpack("C4",pack("N", 2**32 - (2 ** (32-$bits)))));
 }
 
+=item get_community(type,host,ip)
+
+Get Community depending on type (ro,rw).
+If C<get_community> is defined, then get the try to get the community from
+shell-command. If C<get_community> is undefined or nothing
+is returned from the command use C<community> or 
+C<community_rw>.
+
+The command specified in C<get_community> must return in stdout a string like
+
+    community=<list of readonly-communities>
+    setCommunity=<list of write-communities>
+
+Returns Community-List as Array reference
+
+Options:
+    type => 'ro'|'rw' for the type of community
+    host => name of the device
+    ip   => device ip-address
+
+=cut
+
+sub get_community($$;$) {
+    my $type = lc(shift);
+    my $host = shift;
+    my $ip = shift || $host;
+ 
+    my $cmd   = $CONFIG{get_community};
+    my $rcom  = $CONFIG{community};
+    my $rwcom = $CONFIG{community_rw};
+ 
+    if (defined $cmd && length($cmd)) {
+        my @com;
+        # replace variables
+        $cmd =~ s/\%HOST\%?/$host/egi;
+        $cmd =~ s/\%IP\%?/$ip/egi;
+        my $return = `$cmd`;
+        my @lines = split (/\n/,$return);
+        foreach (@lines) {
+            if (/^community\s*=\s*(.*)\s*$/i) {
+                if (length($1)) {
+                    @com = split(/\s*,\s*/,$1);
+                    $rcom = \@com;
+                } else {
+                    $rcom = undef;
+                }
+            } elsif (/^setCommunity\s*=\s*(.*)\s*$/i) {
+                if (length($1)) {
+                    @com = split(/\s*,\s*/,$1);
+                    $rwcom = \@com;
+                } else {
+                    $rwcom = undef;
+                }
+            }
+        }
+    }
+    return  ($type eq 'rw' ? $rwcom : $rcom);
+}
+
 =item  config() 
 
 Reads the config file and fills the C<%CONFIG> hash.
@@ -249,27 +308,34 @@ sub config {
     my @booleans = qw/compresslogs ignore_private_nets reverse_sysname daemon_bg
                       port_info secure_server graph_clusters graph_splines portctl_uplinks
                       portctl_nophones portctl_vlans macsuck_all_vlans macsuck_bleed
-                      bulkwalk_off vlanctl apache_auth nonincreasing
-                     /;
+                      bulkwalk_off vlanctl apache_auth nonincreasing store_modules
+                      vacuum_no store_wireless_client/;
 
     # these will make array refs of their comma separated lists
     my @array_refs = qw/community community_rw mibdirs bulkwalk_no
-                        macsuck_no arpnip_no discover_no
+                        macsuck_no arpnip_no discover_no ignore_interfaces
                         macsuck_only arpnip_only discover_only
-                        snmpforce_v1 snmpforce_v2 snmpforce_v3/;
+                        snmpforce_v1 snmpforce_v2 snmpforce_v3 db_tables
+                        v3_users v3_users_rw
+                        ldap_server /;
 
     # these will make a reference to a hash:
     #      keys :comma separated list entries value : number > 0
     my @hash_refs  = qw/portcontrol admin web_console_vendors
                        web_console_models macsuck_no_vlan
+                       ldap_opts ldap_tls_opts
                       /;
 
     # Multiple arrays
     my @array_refs_mult = qw/node_map/;
 
+    # Reference to a hash; each line is an entry in the hash,
+    #  with format key:value
+    my @hash_refs_mult = qw/v3_user/;
+
     # Add custom types from caller outside netdisco
-    foreach my $type qw(booleans array_refs hash_refs array_refs_mult) {
-	eval "push(\@$type, \@{\$args{config}{\$type}});";
+    foreach my $type qw(booleans array_refs hash_refs array_refs_mult hash_refs_mult) {
+        eval "push(\@$type, \@{\$args{config}{\$type}});";
     }
 
     open(CONF, "<$file") or die "Can't open Config File $file. $!\n";
@@ -280,6 +346,9 @@ sub config {
     foreach my $a (sort @array_refs_mult) {
         $CONFIG{$a} = [];
     }
+    foreach my $a (@hash_refs_mult) {
+        $CONFIG{$a} = {};
+    }
 
     while(my $config = shift @configs){
         chomp $config;
@@ -374,6 +443,13 @@ sub config {
             $value = $oldvalue;
         }
 
+        # Multiple hash refs
+        if (grep /^\Q$var\E$/,@hash_refs_mult) {
+            my ($key, $val) = split(/\s*:\s*/, $value, 2);
+            $value = $CONFIG{$var};
+            $value->{$key} = $val;
+        }
+
         # Comma separated lists that map to defined hash keys.
         if (grep /^\Q$var\E$/,@hash_refs) {
             my %seen;
@@ -422,20 +498,20 @@ config if there's a problem with the config file.
 =cut
 
 sub updateconfig {
-	my $needupdate = 0;
+        my $needupdate = 0;
 
-	if (defined($CONFIG{'@file'}) && defined($CONFIG{'@mtime'})) {
-		if (-M $CONFIG{'@file'} < $CONFIG{'@mtime'}) {
-			$needupdate = 1;
-		}
-	}
-	if ($needupdate) {
-		eval { config($CONFIG{'@file'}); };
-		if ($@) {
-			carp($@);
-		}
-	}
-	$needupdate;
+        if (defined($CONFIG{'@file'}) && defined($CONFIG{'@mtime'})) {
+                if (-M $CONFIG{'@file'} < $CONFIG{'@mtime'}) {
+                        $needupdate = 1;
+                }
+        }
+        if ($needupdate) {
+                eval { config($CONFIG{'@file'}); };
+                if ($@) {
+                        carp($@);
+                }
+        }
+        $needupdate;
 }
 
 =item has_layer(bit string,layer) 
@@ -474,6 +550,9 @@ Returns the IP Address of a given IP or hostname. If the
 given argument appears to be in dotted octet notation, it
 does no DNS hits and just returns it.
 
+It also just returns an IP address with a subnet mask.
+Subnet masks are not permitted on host names.
+
 =cut
 
 sub getip {
@@ -481,10 +560,10 @@ sub getip {
 
     my $ip;
 
-    if ($hostname =~ /^\d+\.\d+\.\d+\.\d+$/) {
+    if ($hostname =~ /^\d+\.\d+\.\d+\.\d+(?:\/\d+)?$/) {
         $ip = $hostname;
     } else {
-        my $testhost = inet_aton($hostname);
+        my $testhost = inet_aton($hostname) || inet_aton($hostname . ($CONFIG{domain} || ''));
         return undef unless (defined $testhost and length $testhost);
         $ip = inet_ntoa($testhost);
     }
@@ -509,20 +588,18 @@ sub in_device {
     return 0 unless defined $to_match;
     return 0 unless defined $device;
 
-    my ($ip,$model,$vendor);
+    my ($ip,$terms);
 
 
     # First Argument:
     # Passed a sql_hash from the device table
     if (ref($device) eq 'HASH'){
         $ip     = $device->{ip};
-        $model  = $device->{model};
-        $vendor = $device->{vendor};
+        $terms  = $device;
     # Passed as simple hostname/IP
     } else {
         $ip     = getip($device);
-        $model  = '';
-        $vendor = '';
+        $terms  = {};
     }
 
     # Second Argument:
@@ -531,14 +608,9 @@ sub in_device {
         $term =~ s/\s*$//;
         # Check for device types
         if ($term =~ /(.*)\s*:\s*(.*)/){
-            my $attrib = $1; my $match = $2;
+            my $attrib = lc($1); my $match = $2;
 
-            if (lc($attrib) eq 'model'){
-                return 1 if $model =~ /^$match$/;
-            }
-            elsif (lc($attrib) eq 'vendor'){
-                return 1 if $vendor =~ /^$match$/;
-            }
+            return 1 if $terms->{$attrib} && $terms->{$attrib} =~ /^$match$/;
             
         # Blanket wildcard
         } elsif ($term eq '*') {
@@ -629,12 +701,12 @@ sub active_subnets {
     my $active = [];
 
     foreach my $row (@$subnets) {
-	my $net = $row->{net};
-	my $found = 0;
-	$found ||= sql_scalar('node_ip',['1 as yup'],{'ip' => \\"<< '$net'"});
-	$found ||= sql_scalar('device',['1 as yup'],{'ip' => \\"<< '$net'"});
-	$found ||= sql_scalar('device_ip',['1 as yup'],{'alias' => \\"<< '$net'"});
-	push(@$active, $net) if $found;
+        my $net = $row->{net};
+        my $found = 0;
+        $found ||= sql_scalar('node_ip',['1 as yup'],{'ip' => \\"<< '$net'"});
+        $found ||= sql_scalar('device',['1 as yup'],{'ip' => \\"<< '$net'"});
+        $found ||= sql_scalar('device_ip',['1 as yup'],{'alias' => \\"<< '$net'"});
+        push(@$active, $net) if $found;
     }
     return $active;
 }
@@ -709,6 +781,8 @@ sub is_mac{
     return 1 if ($mac =~ /^${hex}{6}[:-]{1}${hex}{6}$/);
     #'0800.2b01.0203'
     return 1 if ($mac =~ /^${hex}{4}\.${hex}{4}\.${hex}{4}$/);
+    #'0800-2b01-0203' c/o http://sites.google.com/site/jrbinks/code/netdisco/mac-patches
+    return 1 if ($mac =~ /^${hex}{4}-${hex}{4}-${hex}{4}$/);
     # '08-00-2b-01-02-03','08:00:2b:01:02:03'
     return 1 if ($mac =~ /^${hex}{2}-${hex}{2}-${hex}{2}-${hex}{2}-${hex}{2}-${hex}{2}$/);
     return 1 if ($mac =~ /^${hex}{2}:${hex}{2}:${hex}{2}:${hex}{2}:${hex}{2}:${hex}{2}$/);
@@ -748,6 +822,46 @@ sub mail {
     close (SENDMAIL) or die "Can't send letter. $!\n";
 }
 
+=item is_private(ip)
+
+Returns true if a given IP address is in the RFC1918 private
+address range.
+
+=cut
+
+sub is_private {
+    my ($ip) = @_;
+    my $ignore = 0;
+
+    # Class A Private
+    $ignore++ if $ip =~ /^10\./;
+    # Class B Private
+    $ignore++ if $ip =~ /^172\.(\d+)\./ && ($1 >= 16 && $1 <= 31);
+    # Class C private
+    $ignore++ if $ip =~ /^192\.168\./;
+
+    return $ignore;
+}
+
+=item cidr(ip, mask)
+
+Takes an IP address and netmask and returns the CIDR format
+subnet.
+
+=cut
+
+sub cidr {
+    my ($ip, $mask) = @_;
+    my $bits = mask_to_bits($mask);
+
+    return undef if (!defined($ip) || !defined($mask));
+
+    my $nip = unpack("N", pack("C*", split(/\./, $ip)));
+    my $nmask = unpack("N", pack("C*", split(/\./, $mask)));
+    my $netaddr = join(".", unpack("C*", pack("N", $nip & $nmask)));
+    return "$netaddr/$bits";
+}
+
 =item mask_to_bits(mask)
 
 Takes a netmask and returns the CIDR integer number of bits.
@@ -1118,7 +1232,7 @@ sub root_device {
 
 Adds or changes a user account.
 
-C<%args> can have key values of { pw, admin, port }
+C<%args> can have key values of { pw, admin, port, ldap, full_name, note }
 
 Returns error message if problem.
 
@@ -1144,6 +1258,10 @@ sub user_add {
             $db_args{admin} = ($val =~ /^(1|true|yes|y)$/i) ? 1 : 0;
         }
 
+        if ($arg eq 'ldap') {
+            $db_args{ldap} = ($val =~ /^(1|true|yes|y)$/i) ? 1 : 0;
+        }
+
         if ($arg eq 'port') {
             $db_args{port_control} = ($val =~ /^(1|true|yes|y)$/i) ? 1 : 0;
         }
@@ -1178,6 +1296,109 @@ sub user_del{
     return sql_do(qq/DELETE from users where username=$user/);
 }
 
+=item user_ldap_verify(user,password)
+
+Test a user from netdisco.
+
+Returns 1 if user and password are OK, or undef if error.
+
+=cut
+
+sub user_ldap_verify{
+    tryuse('Net::LDAP', die => 1);
+    my ($user, $pass) = @_;
+
+    return undef unless defined($user);
+
+    my $ldapuser = $CONFIG{ldap_user_string};
+    $ldapuser =~ s/\%USER\%?/$user/egi;
+
+    # If we can bind as anonymous or proxy user
+    # search for user's distinguished name
+    if ( $CONFIG{ldap_proxy_user} ) {
+        my $user   = $CONFIG{ldap_proxy_user};
+        my $pass   = $CONFIG{ldap_proxy_pass};
+        my $attrs  = ['distinguishedName'];
+        my $result = ldap_search ($ldapuser, $attrs, $user, $pass);
+        $ldapuser  = $result->[0] if ($result->[0]);
+    }
+    # If we can't search and aren't using AD and then construct DN by
+    # appending base
+    elsif ( $ldapuser =~ /=/ ) {
+        $ldapuser = "$ldapuser,$CONFIG{ldap_base}";
+    }
+    
+    foreach my $server (@{$CONFIG{ldap_server}}) {
+        my $opts = $CONFIG{ldap_opts} || {};
+        my $ldap = Net::LDAP->new( $server, %$opts ) or next;
+
+        my $msg ;
+
+        if ( $CONFIG{ldap_tls_opts} ) {
+            $msg  = $ldap->start_tls( %{$CONFIG{ldap_tls_opts}} );
+        }
+       
+        $msg = $ldap->bind( $ldapuser, password => $pass );
+
+        $ldap->unbind(); # take down session
+
+        return 1 unless $msg->code();
+    }
+    return undef;
+}
+
+=item ldap_search(filter,attrs,user,pass)
+
+Perform an LDAP search from the configured ldap_base with the specified filter.
+
+Uses an anonymous bind if the user is 'anonymous' or undefined.
+
+Returns reference to an array of Net::LDAP::Entry objects which match the
+search filter.  Each entry will contain the accessible attributes as defined
+in attrs array reference.  If attrs is undefined, then the server will return
+the attributes that are specified as accessible by default given the bind
+credentials.
+
+=cut
+
+sub ldap_search {
+    tryuse('Net::LDAP', die => 1);
+    my ($filter, $attrs, $user, $pass) = @_;
+
+    return undef unless defined($filter);
+    return undef if (defined $attrs and ref($attrs) ne 'ARRAY');
+
+    foreach my $server (@{$CONFIG{ldap_server}}) {
+        my $opts = $CONFIG{ldap_opts} || {};
+        my $ldap = Net::LDAP->new( $server, %$opts ) or next;
+
+        my $msg;
+
+        if ( $CONFIG{ldap_tls_opts} ) {
+            $msg  = $ldap->start_tls( %{$CONFIG{ldap_tls_opts}} );
+        }
+        
+        if ( $user and $user ne 'anonymous' ) {
+            $msg = $ldap->bind( $user, password => $pass);
+        }
+        else {
+            $msg = $ldap->bind();
+        }
+
+        $msg = $ldap->search(   
+                                base=> $CONFIG{ldap_base},
+                                filter=>"($filter)",
+                                attrs=> $attrs );
+
+        $ldap->unbind(); # take down session
+        
+        my $entries = [ $msg->entries ];
+
+        return $entries unless $msg->code();
+    }
+    return undef;
+}
+
 =back
 
 =head2 SQL Functions
@@ -1193,25 +1414,34 @@ Select database handle in use by localizing C<$netdisco::DB>
 
 =cut
 
-sub dbh {
-    unless ($DBH{$DB} && $DBH{$DB}->ping) {
-        my $connect = $CONFIG{"db_$DB"}        or die "dbh() - db_$DB not found in config info.\n";
-        my $user    = $CONFIG{"db_${DB}_user"} or die "dbh() - db_${DB}_user not found in config info.\n";
-        my $pw      = $CONFIG{"db_${DB}_pw"}   or die "dbh() - db_${DB}_pw not found in config info.\n";
-        my $options = $CONFIG{"db_${DB}_opts"} || {};
-        my $env     = $CONFIG{"db_${DB}_env"} || '';
-        # Multiple environmental variables separated by commas.
-        foreach my $e (split(/\s*(?<!\\),\s*/,$env) ) {
-            $e =~ s!\\,!,!g;
-            my ($key,$val) = split(/\s*=>\s*/,$e);
-            next unless (defined $key and defined $val and $key and $val);
-            #warn "Setting ENV{$key} to $val\n";
-            $ENV{$key}=$val;
-        }
-        $DBH{$DB} = DBI->connect($connect,$user,$pw,$options)
-            or die "Can't connect to DB. $DBI::errstr\n";
+sub _make_connection {
+    my $connect = $CONFIG{"db_$DB"}        or die "dbh() - db_$DB not found in config info.\n";
+    my $user    = $CONFIG{"db_${DB}_user"} or die "dbh() - db_${DB}_user not found in config info.\n";
+    my $pw      = $CONFIG{"db_${DB}_pw"}   or die "dbh() - db_${DB}_pw not found in config info.\n";
+    my $options = $CONFIG{"db_${DB}_opts"} || {};
+    my $env     = $CONFIG{"db_${DB}_env"} || '';
+    # Multiple environmental variables separated by commas.
+    foreach my $e (split(/\s*(?<!\\),\s*/,$env) ) {
+        $e =~ s!\\,!,!g;
+        my ($key,$val) = split(/\s*=>\s*/,$e);
+        next unless (defined $key and defined $val and $key and $val);
+        #warn "Setting ENV{$key} to $val\n";
+        $ENV{$key}=$val;
     }
-    return $DBH{$DB}; 
+
+    my $h = DBI->connect($connect,$user,$pw,$options)
+        or die "Can't connect to DB. $DBI::errstr\n";
+    return $h;
+}
+
+sub dbh {
+    unless ($INC{'Apache/DBI.pm'} && $ENV{MOD_PERL}) {
+        unless ($DBH{$DB} && $DBH{$DB}->ping) {
+            $DBH{$DB} = &_make_connection;
+        }
+        return $DBH{$DB};
+    }
+    return &_make_connection;
 }
 
 
@@ -1292,9 +1522,9 @@ sub insert_or_update {
 
         my $wherestr = join(' AND ',@where);
 
-        $sql = qq/SELECT * FROM $table WHERE $wherestr/;
+        $sql = qq/SELECT * FROM $table WHERE $wherestr FOR UPDATE/;
 
-        carp($sql) if $SQLCARP;
+        carp("[$$] $sql") if $SQLCARP;
         my $row = $dbh->selectrow_hashref($sql);
         carp "insert_or_update($sql) ". $dbh->errstr . "\n" if $dbh->err;
 
@@ -1307,7 +1537,7 @@ sub insert_or_update {
 
             # Certain devices were null padding and postgres barfs on nulls in text fields.
             $sql =~ s/\0//g;
-            carp($sql) if $SQLCARP;
+            carp("[$$] $sql") if $SQLCARP;
 
             $dbh->do($sql); 
             if ($dbh->err) { 
@@ -1328,7 +1558,7 @@ sub insert_or_update {
 
     # Certain devices were null padding and postgres barfs on nulls in text fields.
     $sql =~ s/\0//g;
-    carp($sql) if $SQLCARP;
+    carp("[$$] $sql") if $SQLCARP;
 
     my $sth = $dbh->prepare($sql);
     if ($dbh->err) { 
@@ -1341,10 +1571,6 @@ sub insert_or_update {
         return undef;
     }
 
-    if ($DB eq 'Pg'){
-        $rv = $sth->{pg_oid_status} || $rv;
-    }
-    
     $sth->finish;
     return $rv;
 }
@@ -1355,10 +1581,7 @@ Returns reference to hash.  Hash has form C<$hash{key_val}={val_val}>
 
 If multiple matches are found for key_col, only the last one is kept.
 
-Supports
-
-    * (NOT) NULL
-    * Auto-Quoting Values
+Usage is the same as sql_rows() -- See for Usage.
 
     $OldDevices = sql_column('device',['ip','layers']);
 
@@ -1367,35 +1590,8 @@ Creates the hash %$OldDevices where the key is the IP address and the Value is t
 =cut
 
 sub sql_column {
-    my $dbh = &dbh;
-
-    my ($table,$column,$wherehash) = @_;
-
-    my $sql = sprintf("SELECT %s FROM $table", join(',',@$column));
-
-    if (defined $wherehash) {
-        my @where;
-        foreach my $index (keys %$wherehash){
-            my $val = $wherehash->{$index};
-
-            my $con = '=';
-
-            if ($val =~  m/^\s*is\s+(not)?\s*null$/i ){
-               $con = ''; 
-            }
-
-            $val = $dbh->quote($val) if $con;
-            push(@where, sprintf("$index $con $val"));
-        }
-
-        $sql .= sprintf(" WHERE %s", join(' AND ',@where));
-    }
-    
-    carp($sql) if $SQLCARP;
-
     my %hash;
-    my $sth = $dbh->prepare($sql);
-    $sth->execute;
+    my $sth = sql_query(@_);
     my $arrayref =  $sth->fetchall_arrayref;
     foreach my $arr (@$arrayref){
         my $idx = $arr->[0];
@@ -1408,6 +1604,36 @@ sub sql_column {
     return \%hash;
 }
 
+=item sql_columns(table,[key_col,val_col,...],{where}) 
+
+Returns reference to hash.  Hash has form C<$hash{key_val}=hash of all columns}>
+
+If multiple matches are found for key_col, only the last one is kept.
+
+Usage is the same as sql_rows() -- See for Usage.
+
+    $OldDevices = sql_columns('device',['ip','layers','dns']);
+
+Creates the hash %$OldDevices where the key is the IP address and the Value is a hash
+with the keys 'ip', 'layers' and 'dns'.
+
+=cut
+
+sub sql_columns {
+    my $key_col = $_[1]->[0];
+    my $sth = sql_query(@_);
+    my $arrayref =  $sth->fetchall_arrayref({});
+    my %hash;
+    foreach my $h (@$arrayref){
+        my $idx = $h->{$key_col};
+        next unless defined $idx;
+        $hash{$idx} = $h;
+    }
+
+    return \%hash;
+}
+
+
 =item sql_do(sql)
 
 Simple wrapper to C<$dbh-E<gt>do()>
@@ -1420,26 +1646,116 @@ sub sql_do {
     my $sql = shift;
     my $dbh = &dbh;    
     
-    carp($sql) if $SQLCARP;
+    carp("[$$] $sql") if $SQLCARP;
 
     return $dbh->do($sql); 
 }
 
+=item sql_begin()
+
+Start an SQL transaction.
+
+Pass an array reference to the list of tables that should be
+locked in EXCLUSIVE mode for this transaction.  Normally, row
+locking is sufficient, so no tables need be locked.  However,
+netdisco's macsuck and arpnip processes perform statements like
+
+    UPDATE table SET active='f' WHERE (ip|mac)='foo'
+
+If two such statements are executed concurrently for the
+same value of 'foo', they can visit the same rows in different
+order, resulting in a deadlock.  It's tempting to say that
+you can solve this by making the UPDATE only touch one row at
+a time by adding "AND active" to the WHERE clause; however,
+it's possible to get multiple rows with active=true and open
+up the window for deadlock again.  Without a significant
+rewrite, the best option is to lock the appropriate table in
+exclusive mode (which still allows readers, such as the web
+front end, but blocks any inserts or updates).  Since netdisco
+performs updates in bulk, the table will not spend a significant
+amount of time locked.
+
+=cut
+
+sub sql_begin {
+    my $tables = shift;
+    my $dbh = &dbh;    
+    carp "[$$] starting a transaction" if $SQLCARP;
+    
+    $dbh->{AutoCommit} = 0;
+    if (defined($tables)) {
+        foreach my $table (@$tables) {
+            sql_do("LOCK TABLE " . $table . " IN EXCLUSIVE MODE");
+        }
+    }
+}
+
+
+=item sql_commit()
+
+Finish an SQL transaction and return to AutoCommit mode
+
+=cut
+
+sub sql_commit {
+    my $dbh = &dbh;    
+    carp "[$$] completing transaction" if $SQLCARP;
+    if ($dbh->{AutoCommit}) {
+        carp "AutoCommit is already on, is this dbh new?";
+        return;
+    }
+    $dbh->commit();
+    $dbh->{AutoCommit} = 1;
+}
+
+=item sql_rollback()
+
+If an SQL transaction is in progress, roll it back
+and return to AutoCommit mode.  Suitable to be called
+in a generic error situation when you don't know what
+has been done, since it is a noop if there is no transaction
+occurring.
+
+=cut
+
+sub sql_rollback {
+    my $dbh = &dbh;    
+    carp "[$$] rolling back transaction" if $SQLCARP;
+    if ($dbh->{AutoCommit}) {
+        carp "AutoCommit is already on, no transaction in progress?" if $::DEBUG;
+        return;
+    }
+    $dbh->rollback();
+    $dbh->{AutoCommit} = 1;
+}
+
+=item sql_disconnect()
+
+Disconnect from the SQL database, e.g., before forking.
+
+=cut
+
+sub sql_disconnect {
+    my $dbh = &dbh;
+    $dbh->disconnect;
+}
+
 =item sql_hash(table, [columns], {where}) 
 
 Returns reference to hash representing single row.
 
-Internally calls sql_rows() -- See for Usage.
+Usage is the same as sql_rows() -- See for Usage.
 
     my $hashref = sql_hash('device',['ip','ports'], {'ip'=>'127.0.0.1'});
 
 =cut
 
 sub sql_hash {
-    my $results = sql_rows(@_);
-    return undef unless defined $results;
-    return undef unless scalar (@$results);
-    return shift(@$results); 
+    my $sth = sql_query(@_);
+    # If the query failed, return undef.
+    return undef if (!defined($sth));
+    # Otherwise, return the first matching row (or undef)
+    return $sth->fetchrow_hashref();
 }
 
 =item sql_match(text,exact_flag)
@@ -1667,6 +1983,11 @@ sub sql_query {
                 } elsif (ref $value eq 'ARRAY'){
                     # Let's not modify passed argument.
                     my @val_copy = @$value;
+                    # Empty list?  Instead of forming a malformed
+                    # query, just return nothing.
+                    if (!@val_copy) {
+                        return undef;
+                    }
                     $con = 'IN';
                     my $newvalue = "(";
                     foreach my $inval (@val_copy){
@@ -1723,7 +2044,7 @@ sub sql_query {
     # Certain devices were null padding and postgres barfs on nulls in text fields.
     $sql =~ s/\0//g;
 
-    carp($sql) if $SQLCARP;
+    carp("[$$] $sql") if $SQLCARP;
 
     my $sth = $dbh->prepare($sql);
     if (!defined $sth) {
@@ -1776,6 +2097,8 @@ sub sql_vacuum{
     my $table = shift;
     my %opts  = @_;
     my $print = (defined $opts{print} and $opts{print}) ? 1 : 0;
+    
+    return 1 if $CONFIG{'vacuum_no'};
 
     # no rated r allowed
     if ($DB eq 'Pg'){
@@ -1872,24 +2195,24 @@ sub tryuse($%) {
     my $msg = undef;
 
     if (defined($tryuseok{$mod})) {
-	($ok, $msg) = @{$tryuseok{$mod}};
+        ($ok, $msg) = @{$tryuseok{$mod}};
     } else {
-	if (defined($args{ver})) {
-	    eval "use $mod $args{ver}";
-	} else {
-	    eval "use $mod";
-	}
-	if ($@) {
-	    $msg = "You need the $mod perl module";
-	    if ($args{ver}) {
-		$msg .= ", version $args{ver} or newer";
-	    }
-	    $msg .= ".\nPlease install it and try again.\n\n" . $@;
-	    $ok = 0;
-	}
+        if (defined($args{ver})) {
+            eval "use $mod $args{ver}";
+        } else {
+            eval "use $mod";
+        }
+        if ($@) {
+            $msg = "You need the $mod perl module";
+            if ($args{ver}) {
+                $msg .= ", version $args{ver} or newer";
+            }
+            $msg .= ".\nPlease install it and try again.\n\n" . $@;
+            $ok = 0;
+        }
     }
     if ($ok == 0 && $args{die}) {
-	die $msg;
+        die $msg;
     }
     my $rv = [ $ok, $msg ];
     $tryuseok{$mod} = $rv;
