@@ -1,5 +1,5 @@
 -- admin table - Queue for admin tasks sent from front-end for back-end processing.
--- $Id: admin.sql,v 1.4 2006/01/16 00:17:54 maxbaker Exp $ 
+-- $Id: admin.sql,v 1.6 2007/10/12 05:31:41 maxbaker Exp $ 
 
 DROP TABLE admin;
 DROP SEQUENCE admin_job_seq;
@@ -19,3 +19,7 @@ CREATE TABLE admin (
     log         text,
     debug       boolean
                    );
+
+CREATE INDEX idx_admin_entered ON admin(entered);
+CREATE INDEX idx_admin_status  ON admin(status);
+CREATE INDEX idx_admin_action  ON admin(action);
